@@ -1,25 +1,27 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #include	"bstpatterneditor.h"
 
+#include	"bstgconfig.h"
 #include	<gtk/gtksignal.h>
 #include	<gdk/gdkkeysyms.h>
 #include	<string.h>
+#include	<ctype.h>
 
 
 /* FIXME: the layout needs to be changed to something like:
@@ -99,7 +101,6 @@
 
 
 /* --- defines --- */
-#define	SELECTION_TIMEOUT		(33)
 #define PE(pe)				((BstPatternEditor*) (pe))
 #define	ALLOCATION(w)			(&((GtkWidget*) (w))->allocation)
 #define	X_THICK(w)			(((GtkWidget*) (w))->style->klass->xthickness)
@@ -264,17 +265,14 @@ static void bst_pattern_editor_channel_popup	(BstPatternEditor	*pe,
 static void bst_pattern_editor_release_pattern	(BstPatternEditor	*pe);
 static void adjustments_value_changed		(GtkAdjustment		*adjustment,
 						 BstPatternEditor	*pe);
-static void bst_pattern_editor_selection_update (BstPatternEditor *pe,
-						 guint		   channel,
-						 guint		   row,
-						 gint		   panel_sa_x,
-						 gint		   panel_sa_y,
-						 gboolean	   keep_selection,
-						 gboolean	   subtract);
-static void bst_pattern_editor_selection_moved	(BstPatternEditor *pe,
-						 gint		   panel_sa_x,
-						 gint		   panel_sa_y);
-static void bst_pattern_editor_selection_done	(BstPatternEditor *pe);
+static void bst_pe_note_changed                 (BstPatternEditor       *pe,
+						 guint	                 channel,
+						 guint	                 row,
+						 BsePattern	        *pattern);
+#define NOTE_CHANGED(pe, channel, row)          (bst_pe_note_changed ((pe), (channel), (row), (pe)->pattern))
+
+
+#include	"bstpatterneditorselection.c"    /* selection functionality */
 
 
 /* --- static variables --- */
@@ -462,13 +460,8 @@ bst_pattern_editor_init (BstPatternEditor *pe)
   pe->next_moves_right = FALSE;
   pe->next_moves_up = FALSE;
   pe->next_moves_down = TRUE;
-  
-  pe->in_selection = FALSE;
-  pe->selection_subtract = FALSE;
-  pe->saved_selection = NULL;
-  pe->selection_channel = 0;
-  pe->selection_row = 0;
-  pe->selection_timer = 0;
+
+  bst_pattern_editor_selection_meminit (pe);
   
   pe->focus_channel = 0;
   pe->focus_row = 0;
@@ -505,7 +498,7 @@ bst_pattern_editor_shutdown (GtkObject *object)
   
   if (pe->pattern)
     bst_pattern_editor_release_pattern (pe);
-  
+
   GTK_OBJECT_CLASS (parent_class)->shutdown (object);
 }
 
@@ -728,6 +721,9 @@ bst_pe_size_changed (BstPatternEditor *pe)
   
   g_return_if_fail (BST_IS_PATTERN_EDITOR (pe));
   
+  if (pe->in_selection)
+    bst_pattern_editor_selection_done (pe);
+  
   focus_channel = pe->focus_channel;
   focus_row = pe->focus_row;
   
@@ -742,7 +738,7 @@ bst_pe_size_changed (BstPatternEditor *pe)
   
   gtk_widget_queue_resize (GTK_WIDGET (pe));
   
-  bst_pattern_editor_set_focus (pe, focus_channel, focus_row);
+  bst_pattern_editor_set_focus (pe, focus_channel, focus_row, FALSE);
 }
 
 static void
@@ -750,26 +746,25 @@ bst_pattern_editor_fetch_pattern_sibling (BstPatternEditor *pe,
 					  BseItem	   *container,
 					  BsePattern	   *pattern)
 {
+  BseItem *item;
   guint seqid;
   
   g_return_if_fail (BST_IS_PATTERN_EDITOR (pe));
   g_return_if_fail (BSE_IS_PATTERN (pattern));
   g_return_if_fail (pe->pattern == pattern);
-  
+
   seqid = bse_item_get_seqid (BSE_ITEM (pattern));
-  container = BSE_ITEM (pattern)->container;
-  if (container && seqid)
-    {
-      BseItem *item;
-      
-      item = bse_container_get_item (BSE_CONTAINER (container), BSE_TYPE_PATTERN, seqid + 1);
-      if (!item && seqid > 1)
-	item = bse_container_get_item (BSE_CONTAINER (container), BSE_TYPE_PATTERN, seqid - 1);
-      if (!item)
-	bst_pattern_editor_release_pattern (pe);
-      else
-	bst_pattern_editor_set_pattern (pe, BSE_PATTERN (item));
-    }
+  container = BSE_ITEM (pattern)->parent;
+
+  g_return_if_fail (container != NULL);
+
+  item = bse_container_get_item (BSE_CONTAINER (container), BSE_TYPE_PATTERN, seqid + 1);
+  if (!item && seqid > 1)
+    item = bse_container_get_item (BSE_CONTAINER (container), BSE_TYPE_PATTERN, seqid - 1);
+  if (!item)
+    bst_pattern_editor_release_pattern (pe);
+  else
+    bst_pattern_editor_set_pattern (pe, BSE_PATTERN (item));
 }
 
 static void
@@ -819,7 +814,7 @@ bst_pattern_editor_set_pattern (BstPatternEditor *pe,
 				    bst_pattern_editor_release_pattern,
 				    pe);
       bse_object_add_data_notifier (pattern,
-				    "set_container",
+				    "set_parent",
 				    bst_pattern_editor_fetch_pattern_sibling,
 				    pe);
       bse_object_add_data_notifier (pattern,
@@ -831,11 +826,7 @@ bst_pattern_editor_set_pattern (BstPatternEditor *pe,
 				    bst_pe_note_changed,
 				    pe);
       bse_object_add_data_notifier (pattern,
-				    "note_selected",
-				    bst_pe_note_changed,
-				    pe);
-      bse_object_add_data_notifier (pattern,
-				    "note_unselected",
+				    "note_selection_changed",
 				    bst_pe_note_changed,
 				    pe);
       bse_object_add_data_notifier (pattern,
@@ -1739,7 +1730,7 @@ bst_pattern_editor_draw_grid (BstPatternEditor *pe,
 			      guint		last_row)
 {
   GtkWidget *widget = GTK_WIDGET (pe);
-  GdkGC *spaced_gc, *dark_gc, *focus_gc;
+  GdkGC *spaced_gc, *dark_gc, *focus_gc, *unfocus_gc;
   guint c, r, i;
   
   g_return_if_fail (first_channel < N_CHANNELS (pe));
@@ -1750,6 +1741,7 @@ bst_pattern_editor_draw_grid (BstPatternEditor *pe,
   spaced_gc = widget->style->base_gc[GTK_WIDGET_STATE (pe)];
   dark_gc = widget->style->dark_gc[GTK_WIDGET_STATE (pe)];
   focus_gc = widget->style->fg_gc[GTK_STATE_PRELIGHT];
+  unfocus_gc = widget->style->bg_gc[GTK_WIDGET_STATE (pe)];
   
   /* draw spaced lines
    */
@@ -1815,36 +1807,39 @@ bst_pattern_editor_draw_grid (BstPatternEditor *pe,
   
   /* draw focus
    */
-  if (GTK_WIDGET_HAS_FOCUS (pe))
+  if (TRUE /* GTK_WIDGET_HAS_FOCUS (pe) */)
     {
+      GdkGC *draw_gc;
       guint x = pe->focus_channel * (GRID_BORDER + TONE_WIDTH (pe));
       guint y = pe->focus_row * (GRID_BORDER + TONE_HEIGHT (pe));
+
+      draw_gc = GTK_WIDGET_HAS_FOCUS (pe) ? focus_gc : unfocus_gc;
       
       for (i = 0; i < GRID_BORDER; i++)
 	{
-	  gdk_draw_line (pe->panel, focus_gc,
+	  gdk_draw_line (pe->panel, draw_gc,
 			 x + i,
 			 y,
 			 x + i,
 			 y + GRID_BORDER + TONE_HEIGHT (pe) + GRID_BORDER - 1);
-	  gdk_draw_line (pe->panel, focus_gc,
+	  gdk_draw_line (pe->panel, draw_gc,
 			 x + GRID_BORDER + TONE_WIDTH (pe) + i,
 			 y,
 			 x + GRID_BORDER + TONE_WIDTH (pe) + i,
 			 y + GRID_BORDER + TONE_HEIGHT (pe) + GRID_BORDER - 1);
-	  gdk_draw_line (pe->panel, focus_gc,
+	  gdk_draw_line (pe->panel, draw_gc,
 			 x + GRID_BORDER,
 			 y + i,
 			 x + GRID_BORDER + TONE_WIDTH (pe) - 1,
 			 y + i);
-	  gdk_draw_line (pe->panel, focus_gc,
+	  gdk_draw_line (pe->panel, draw_gc,
 			 x + GRID_BORDER,
 			 y + GRID_BORDER + TONE_HEIGHT (pe) + i,
 			 x + GRID_BORDER + TONE_WIDTH (pe) - 1,
 			 y + GRID_BORDER + TONE_HEIGHT (pe) + i);
 	}
       if (GRID_BORDER < 2) /* hm, hack alert */
-	gdk_draw_rectangle (pe->panel, focus_gc, FALSE,
+	gdk_draw_rectangle (pe->panel, draw_gc, FALSE,
 			    TONE_X (pe, pe->focus_channel),
 			    TONE_Y (pe, pe->focus_row),
 			    TONE_WIDTH (pe) - 1,
@@ -1858,16 +1853,18 @@ bst_pattern_editor_draw_tone (BstPatternEditor *pe,
 			      guint		row)
 {
   GtkWidget *widget = GTK_WIDGET (pe);
-  BseNote *note;
+  BsePatternNote *note;
   guint tone_x, tone_y, tone_width, tone_height;
-  GdkGC *fg_gc, *bg_gc;
+  GdkGC *fg_gc, *bg_gc, *light_gc;
   gchar buffer[64], *p;
+  gboolean selected;
   
   g_return_if_fail (channel < N_CHANNELS (pe));
   g_return_if_fail (row < N_ROWS (pe));
   
   note = bse_pattern_peek_note (pe->pattern, channel, row);
-  if (note->selected)
+  selected = note->selected;
+  if (selected)
     {
       fg_gc = widget->style->fg_gc[GTK_STATE_SELECTED];
       bg_gc = widget->style->bg_gc[GTK_STATE_SELECTED];
@@ -1877,6 +1874,7 @@ bst_pattern_editor_draw_tone (BstPatternEditor *pe,
       fg_gc = widget->style->fg_gc[GTK_WIDGET_STATE (pe)];
       bg_gc = widget->style->base_gc[GTK_WIDGET_STATE (pe)];
     }
+  light_gc = widget->style->dark_gc[GTK_WIDGET_STATE (pe)];
   
   /* base allocation
    */
@@ -1887,7 +1885,7 @@ bst_pattern_editor_draw_tone (BstPatternEditor *pe,
   
   /* clear/paint area
    */
-  if (note->selected)
+  if (selected)
     gdk_draw_rectangle (pe->panel, bg_gc, TRUE,
 			tone_x, tone_y,
 			tone_width, tone_height);
@@ -1914,7 +1912,7 @@ bst_pattern_editor_draw_tone (BstPatternEditor *pe,
     g_snprintf (buffer, 64, NOTE_EMPTY);
   gdk_draw_string (pe->panel,
 		   widget->style->font,
-		   fg_gc,
+		   note->note != BSE_NOTE_VOID ? fg_gc : light_gc,
 		   tone_x + NOTE_X (pe),
 		   tone_y + NOTE_Y (pe),
 		   buffer);
@@ -1933,7 +1931,7 @@ bst_pattern_editor_draw_tone (BstPatternEditor *pe,
     *(p++) = '-';
   gdk_draw_string (pe->panel,
 		   widget->style->font,
-		   fg_gc,
+		   note->instrument ? fg_gc : light_gc,
 		   tone_x + INSTRUMENT_X (pe),
 		   tone_y + INSTRUMENT_Y (pe),
 		   buffer);
@@ -1972,7 +1970,8 @@ bst_pattern_editor_set_octave (BstPatternEditor *pe,
 void
 bst_pattern_editor_set_focus (BstPatternEditor *pe,
 			      guint		channel,
-			      guint		row)
+			      guint		row,
+			      gboolean          reset_selection)
 {
   GtkWidget *widget;
   guint old_channel, old_row;
@@ -1992,251 +1991,26 @@ bst_pattern_editor_set_focus (BstPatternEditor *pe,
   pe->focus_channel = channel;
   pe->focus_row = row;
   
-  if ((old_channel != pe->focus_channel ||
-       old_row != pe->focus_row) &&
-      GTK_WIDGET_DRAWABLE (pe))
+  if (old_channel != pe->focus_channel ||
+      old_row != pe->focus_row)
     {
-      if (old_channel < N_CHANNELS (pe) &&
-	  old_row < N_ROWS (pe))
+      if (GTK_WIDGET_DRAWABLE (pe))
 	{
-	  bst_pattern_editor_draw_tone (pe, old_channel, old_row);
-	  bst_pattern_editor_draw_grid (pe, old_channel, old_row, 0, 0);
-	}
-      bst_pattern_editor_draw_grid (pe, pe->focus_channel, pe->focus_row, 0, 0);
-      bst_pattern_editor_draw_grid (pe, pe->focus_channel, pe->focus_row, 0, 0);
-    }
-  
-  bst_pattern_editor_adjust_sas (pe, FALSE);
-}
-
-static guint32*
-save_selection (BstPatternEditor *pe,
-		gboolean	  keep_selection)
-{
-  guint32 *barray;
-  
-  barray = g_new0 (guint32, (N_CHANNELS (pe) * N_ROWS (pe) + 31) / 32);
-  if (keep_selection)
-    {
-      guint c, r;
-      
-      for (c = 0; c < N_CHANNELS (pe); c++)
-	for (r = 0; r < N_ROWS (pe); r++)
-	  {
-	    BseNote *note = bse_pattern_peek_note (pe->pattern, c, r);
-	    
-	    if (note->selected)
-	      {
-		guint n = N_CHANNELS (pe) * r + c;
-		
-		barray[n / 32] |= 1 << n % 32;
-	      }
-	  }
-    }
-  
-  return barray;
-}
-
-static inline guint
-test_selection (BstPatternEditor *pe,
-		guint		  channel,
-		guint		  row)
-{
-  guint n = N_CHANNELS (pe) * row + channel;
-  
-  return pe->saved_selection[n / 32] & 1 << n % 32;
-}
-
-static void
-bst_pattern_editor_selection_update (BstPatternEditor *pe,
-				     guint	       channel,
-				     guint	       row,
-				     gint	       panel_sa_x,
-				     gint	       panel_sa_y,
-				     gboolean	       keep_selection,
-				     gboolean	       subtract)
-{
-  guint r, c;
-  gboolean selection_started = FALSE;
-  
-  channel = MIN (channel, N_CHANNELS (pe) - 1);
-  row = MIN (row, N_ROWS (pe) - 1);
-  
-  if (!pe->in_selection)
-    {
-      GdkCursor *cursor = gdk_cursor_new (GDK_FLEUR);
-      gboolean failed = gdk_pointer_grab (pe->panel_sa, FALSE,
-					  GDK_BUTTON_PRESS_MASK |
-					  GDK_BUTTON_RELEASE_MASK |
-					  GDK_POINTER_MOTION_MASK |
-					  GDK_POINTER_MOTION_HINT_MASK,
-					  NULL,
-					  cursor,
-					  GDK_CURRENT_TIME);
-      gdk_cursor_destroy (cursor);
-      if (failed)
-	{
-	  gdk_beep ();
-	  return;
-	}
-      
-      bse_object_lock (BSE_OBJECT (pe->pattern));
-      
-      pe->in_selection = TRUE;
-      pe->selection_subtract = subtract != FALSE;
-      pe->saved_selection = save_selection (pe, keep_selection);
-      pe->selection_timer = 0;
-      selection_started = TRUE;
-    }
-  
-  if (pe->selection_channel != channel ||
-      pe->selection_row != row ||
-      selection_started)
-    {
-      guint b_c = MIN (pe->selection_channel, channel);
-      guint b_r = MIN (pe->selection_row, row);
-      guint e_c = MAX (pe->selection_channel, channel);
-      guint e_r = MAX (pe->selection_row, row);
-      
-      if (selection_started)
-	{
-	  b_c = 0;
-	  b_r = 0;
-	  e_c = N_CHANNELS (pe) - 1;
-	  e_r = N_ROWS (pe) - 1;
-	}
-      else
-	{
-	  b_c = MIN (pe->focus_channel, b_c);
-	  b_r = MIN (pe->focus_row, b_r);
-	  e_c = MAX (pe->focus_channel, e_c);
-	  e_r = MAX (pe->focus_row, e_r);
-	}
-      
-      pe->selection_channel = channel;
-      pe->selection_row = row;
-      
-      for (c = b_c; c <= e_c; c++)
-	for (r = b_r; r <= e_r; r++)
-	  {
-	    BseNote *note = bse_pattern_peek_note (pe->pattern, c, r);
-	    gboolean want_selection;
-	    gboolean in_selection = (c >= MIN (pe->focus_channel, pe->selection_channel) &&
-				     c <= MAX (pe->focus_channel, pe->selection_channel) &&
-				     r >= MIN (pe->focus_row, pe->selection_row) &&
-				     r <= MAX (pe->focus_row, pe->selection_row));
-	    
-	    if (pe->selection_subtract)
-	      want_selection = !in_selection && test_selection (pe, c, r);
-	    else
-	      want_selection = in_selection || test_selection (pe, c, r);
-	    
-	    if (want_selection && !note->selected)
-	      bse_pattern_select_note (pe->pattern, c, r);
-	    else if (!want_selection && note->selected)
-	      bse_pattern_unselect_note (pe->pattern, c, r);
-	  }
-      
-      bst_pattern_editor_adjust_sas (pe, FALSE);
-      bst_pattern_editor_selection_moved (pe, panel_sa_x, panel_sa_y);
-    }
-}
-
-static gint
-selection_timeout (gpointer data)
-{
-  BstPatternEditor *pe = BST_PATTERN_EDITOR (data);
-  
-  pe->selection_timer = 0;
-  
-  if (pe->in_selection)
-    {
-      gint x, y;
-      
-      gdk_window_get_pointer (pe->panel_sa, &x, &y, NULL);
-      bst_pattern_editor_selection_update (pe,
-					   pe->selection_timer_channel,
-					   pe->selection_timer_row,
-					   x, y,
-					   0, 0);
-    }
-  
-  return FALSE;
-}
-
-static void
-bst_pattern_editor_selection_moved (BstPatternEditor *pe,
-				    gint	      panel_sa_x,
-				    gint	      panel_sa_y)
-{
-  if (pe->in_selection)
-    {
-      gint channel, row;
-      gint sa_x, sa_y, sa_width, sa_height;
-      BstCellType cell_type;
-      
-      gdk_window_get_position (pe->panel, &sa_x, &sa_y);
-      gdk_window_get_size (pe->panel_sa, &sa_width, &sa_height);
-      
-      if (bst_pattern_editor_get_cell (pe,
-				       panel_sa_x - sa_x,
-				       panel_sa_y - sa_y,
-				       &cell_type,
-				       &channel, &row) &&
-	  cell_type &&
-	  panel_sa_x > 0 && panel_sa_x < sa_width &&
-	  panel_sa_y > 0 && panel_sa_y < sa_height)
-	{
-	  if (pe->selection_timer)
+	  if (old_channel < N_CHANNELS (pe) &&
+	      old_row < N_ROWS (pe))
 	    {
-	      gtk_timeout_remove (pe->selection_timer);
-	      pe->selection_timer = 0;
+	      bst_pattern_editor_draw_tone (pe, old_channel, old_row);
+	      bst_pattern_editor_draw_grid (pe, old_channel, old_row, 0, 0);
 	    }
-	  
-	  bst_pattern_editor_selection_update (pe,
-					       channel, row,
-					       panel_sa_x, panel_sa_y,
-					       0, 0);
-	}
-      else if (panel_sa_x < 0 || panel_sa_x > sa_width ||
-	       panel_sa_y < 0 || panel_sa_y > sa_height)
-	{
-	  pe->selection_timer_channel = MAX (0, channel);
-	  pe->selection_timer_row = MAX (0, row);
-	  
-	  if (!pe->selection_timer)
-	    pe->selection_timer = gtk_timeout_add (SELECTION_TIMEOUT,
-						   selection_timeout,
-						   pe);
+	  bst_pattern_editor_draw_tone (pe, pe->focus_channel, pe->focus_row);
+	  bst_pattern_editor_draw_grid (pe, pe->focus_channel, pe->focus_row, 0, 0);
 	}
     }
-}
 
-static void
-bst_pattern_editor_selection_done (BstPatternEditor *pe)
-{
-  if (pe->in_selection)
-    {
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
-      
-      g_free (pe->saved_selection);
-      pe->saved_selection = NULL;
-      
-      pe->selection_subtract = FALSE;
-      pe->selection_channel = 0;
-      pe->selection_row = 0;
-      pe->in_selection = FALSE;
-      
-      if (pe->selection_timer)
-	{
-	  gtk_timeout_remove (pe->selection_timer);
-	  pe->selection_timer = 0;
-	}
-      
-      bse_object_unlock (BSE_OBJECT (pe->pattern));
-      
-      /* bst_pattern_editor_adjust_sas (pe, FALSE); */
-    }
+  if (reset_selection)
+    bst_pattern_editor_reset_selection (pe);
+
+  bst_pattern_editor_adjust_sas (pe, FALSE);
 }
 
 static gint
@@ -2255,11 +2029,11 @@ bst_pattern_editor_motion (GtkWidget	  *widget,
 	  gint x, y;
 	  
 	  gdk_window_get_pointer (pe->panel_sa, &x, &y, NULL);
-	  bst_pattern_editor_selection_update (pe,
-					       channel, row,
-					       x, y,
-					       event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK),
-					       event->state & GDK_CONTROL_MASK);
+	  bst_pattern_editor_selection_start (pe,
+					      channel, row,
+					      x, y,
+					      event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK),
+					      event->state & GDK_CONTROL_MASK);
 	}
       
       handled = TRUE;
@@ -2269,7 +2043,7 @@ bst_pattern_editor_motion (GtkWidget	  *widget,
       gint x, y;
       
       gdk_window_get_pointer (pe->panel_sa, &x, &y, NULL);
-      bst_pattern_editor_selection_moved (pe, x, y);
+      bst_pattern_editor_selection_motion (pe, x, y);
       
       handled = TRUE;
     }
@@ -2302,17 +2076,18 @@ bst_pattern_editor_button_press (GtkWidget	*widget,
 					       &focus_channel, &focus_row))
 	{
 	  if (event->button == 1)
-	    bst_pattern_editor_set_focus (pe, focus_channel, focus_row);
-	  if (cell_type > BST_CELL_INVALID)
-	    gtk_signal_emit (GTK_OBJECT (pe),
-			     pe_signals[SIGNAL_CELL_CLICKED],
-			     focus_channel,
-			     focus_row,
-			     cell_type,
-			     (guint) event->x_root,
-			     (guint) event->y_root,
-			     (guint) event->button,
-			     (guint) event->time);
+	    bst_pattern_editor_set_focus (pe,
+					  focus_channel, focus_row,
+					  !(event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)));
+	  gtk_signal_emit (GTK_OBJECT (pe),
+			   pe_signals[SIGNAL_CELL_CLICKED],
+			   focus_channel,
+			   focus_row,
+			   cell_type,
+			   (guint) event->x_root,
+			   (guint) event->y_root,
+			   (guint) event->button,
+			   (guint) event->time);
 	}
       handled = TRUE;
     }
@@ -2339,63 +2114,32 @@ bst_pattern_editor_button_release (GtkWidget	  *widget,
   
   if (event->button == 1)
     {
-      bst_pattern_editor_selection_done (pe);
+      if (pe->in_selection)
+	bst_pattern_editor_selection_done (pe);
       handled = TRUE;
     }
   
   return handled;
 }
 
-void
-bst_pattern_editor_class_set_key (BstPatternEditorClass	*pe_class,
-				  guint16		 keyval,
-				  guint16		 modifier,
-				  BstPEActionType	 pe_action)
-{
-  g_return_if_fail (BST_IS_PATTERN_EDITOR_CLASS (pe_class));
-  
-  g_hash_table_insert (pe_class->pea_ktab,
-		       (gpointer) (keyval | ((modifier & BST_MOD_SCA) << 16)),
-		       (gpointer) (pe_action | BST_PEA_TAG));
-}
-
-static gboolean
-bst_clear_keys (gpointer key,
-		gpointer value,
-		gpointer user_data)
-{
-  return TRUE;
-}
-
-void
-bst_pattern_editor_class_clear_keys (BstPatternEditorClass *pe_class)
-{
-  g_return_if_fail (BST_IS_PATTERN_EDITOR_CLASS (pe_class));
-  
-  g_hash_table_foreach_remove (pe_class->pea_ktab,
-			       bst_clear_keys,
-			       NULL);
-}
-
 static gint
 bst_pattern_editor_key_press (GtkWidget	  *widget,
 			      GdkEventKey *event)
 {
   BstPatternEditor *pe;
-  BstPEActionType pea;
+  BstPEActionType pea = 0;
   guint16 modifier;
   guint16 masks[] = {
     BST_MOD_SCA, BST_MOD_SC0, BST_MOD_S0A, BST_MOD_S00,
     BST_MOD_0CA, BST_MOD_0C0, BST_MOD_00A, BST_MOD_000,
   };
   guint n_masks = sizeof (masks) / sizeof (masks[0]);
-  gpointer p;
   guint i;
-  BseNote *bnote;
+  BsePatternNote *pnote;
   BseInstrument *instrument;
   gint focus_channel;
   gint focus_row;
-  guint note;
+  gint note;
   gint difference;
   guint new_focus_channel;
   guint new_focus_row;
@@ -2406,23 +2150,25 @@ bst_pattern_editor_key_press (GtkWidget	  *widget,
   pe = BST_PATTERN_EDITOR (widget);
   
   modifier = event->state & BST_MOD_SCA;
-  p = 0;
-  for (i = 0; !(((guint32) p) & BST_PEA_TAG) && i < n_masks; i++)
+  for (i = 0; !(pea & BST_PEA_TAG) && i < n_masks; i++)
     if (modifier == masks[i])
-      p = g_hash_table_lookup (BST_PATTERN_EDITOR_CLASS (((GtkObject*) pe)->klass)->pea_ktab,
-			       (gpointer) (event->keyval | ((modifier & masks[i]) << 16)));
+      {
+	gpointer p = g_hash_table_lookup (BST_PATTERN_EDITOR_GET_CLASS (pe)->pea_ktab,
+					  GUINT_TO_POINTER (event->keyval | ((modifier & masks[i]) << 16)));
+	
+	pea = GPOINTER_TO_UINT (p);
+      }
   
   focus_channel = pe->focus_channel;
   focus_row = pe->focus_row;
-  bnote = bse_pattern_peek_note (pe->pattern, focus_channel, focus_row);
-  note = bnote->note;
-  instrument = bnote->instrument;
+  pnote = bse_pattern_peek_note (pe->pattern, focus_channel, focus_row);
+  note = pnote->note;
+  instrument = pnote->instrument;
   
   difference = 0;
   new_focus_channel = 0;
   new_focus_row = 0;
   
-  pea = (BstPEActionType) p;
   if (pea & BST_PEA_TAG)
     {
       BstPEActionType wrap;
@@ -2597,12 +2343,12 @@ bst_pattern_editor_key_press (GtkWidget	  *widget,
 	}
     }
   
-  if (note != bnote->note)
+  if (note != pnote->note)
     bse_pattern_set_note (pe->pattern,
 			  pe->focus_channel,
 			  pe->focus_row,
 			  note);
-  if (instrument != bnote->instrument)
+  if (instrument != pnote->instrument)
     bse_pattern_set_instrument (pe->pattern,
 				pe->focus_channel,
 				pe->focus_row,
@@ -2610,7 +2356,7 @@ bst_pattern_editor_key_press (GtkWidget	  *widget,
   
   if (focus_channel != pe->focus_channel ||
       focus_row != pe->focus_row)
-    bst_pattern_editor_set_focus (pe, focus_channel, focus_row);
+    bst_pattern_editor_set_focus (pe, focus_channel, focus_row, BST_PE_KEY_FOCUS_UNSELECTS);
   
   if (difference != 0)
     {
@@ -2625,7 +2371,7 @@ bst_pattern_editor_key_press (GtkWidget	  *widget,
 	{
 	  if (new_focus_channel != pe->focus_channel ||
 	      new_focus_row != pe->focus_row)
-	    bst_pattern_editor_set_focus (pe, new_focus_channel, new_focus_row);
+	    bst_pattern_editor_set_focus (pe, new_focus_channel, new_focus_row, BST_PE_KEY_FOCUS_UNSELECTS);
 	}
     }
   
@@ -2723,7 +2469,7 @@ bst_pattern_editor_channel_popup (BstPatternEditor *pe,
       g_snprintf (buffer, 64, INSTRUMENT_FMT, bse_item_get_seqid (BSE_ITEM (instrument)));
       string = BSE_OBJECT_NAME (instrument);
       if (!string || *string == 0)
-	string = BSE_OBJECT_NAME (instrument->sample);
+	string = BSE_OBJECT_NAME (instrument->input);
       string = g_strconcat (buffer, ") ", string, NULL);
       item = gtk_menu_item_new_with_label (string);
       gtk_widget_set (item,
@@ -2801,3 +2547,179 @@ bst_pattern_editor_set_effect_hooks (BstPatternEditor		 *pe,
   pe->ea_data = user_data;
   pe->ea_destroy = ea_destroy;
 }
+
+void
+bst_pattern_editor_class_set_key (BstPatternEditorClass	*class,
+				  guint16		 keyval,
+				  guint16		 modifier,
+				  BstPEActionType	 pe_action)
+{
+  g_return_if_fail (BST_IS_PATTERN_EDITOR_CLASS (class));
+  
+  g_hash_table_insert (class->pea_ktab,
+		       GUINT_TO_POINTER (keyval | ((modifier & BST_MOD_SCA) << 16)),
+		       GUINT_TO_POINTER (pe_action | BST_PEA_TAG));
+}
+
+void
+bst_pattern_editor_class_clear_keys (BstPatternEditorClass *class)
+{
+  g_return_if_fail (BST_IS_PATTERN_EDITOR_CLASS (class));
+  
+  g_hash_table_foreach_remove (class->pea_ktab,
+			       (GHRFunc) gtk_true,
+			       NULL);
+}
+
+static void
+string_dump_pea (gpointer key,
+		 gpointer value,
+		 gpointer user_data)
+{
+  GSList **slist_p = user_data;
+  BstPEActionType pea = GPOINTER_TO_UINT (value);
+  guint keyval = GPOINTER_TO_UINT (key) & 0xffff;
+  guint modifier = GPOINTER_TO_UINT (key) >> 16;
+  GString *gstring = g_string_new ("");
+  gchar *name;
+  
+  name = gdk_keyval_name (gdk_keyval_to_lower (keyval));
+  
+  /* modifier + keyvalname
+   */
+  g_string_sprintfa (gstring, "%-12.12s ", name);
+  g_string_sprintfa (gstring, "%c%c%c ",
+		     modifier & BST_MOD_S00 ? 'S' : '_',
+		     modifier & BST_MOD_0C0 ? 'C' : '_',
+		     modifier & BST_MOD_00A ? 'A' : '_');
+  
+  /* note
+   */
+  name = bse_note_to_string (BSE_NOTE_C (0) + (pea & BST_PEA_NOTE_MASK));
+  name[0] = toupper (name[0]);
+  g_string_sprintfa (gstring, "%-3.3s ", name[0] != 'v' ? name : "");
+  g_free (name);
+
+  /* octave
+   */
+  switch (pea & BST_PEA_MOVE_MASK)
+    {
+    case BST_PEA_OCTAVE_SHIFT_UP:	name = "+1";	break;
+    case BST_PEA_OCTAVE_SHIFT_DOWN:	name = "-1";	break;
+    case BST_PEA_OCTAVE_SHIFT_UP2:	name = "+2";	break;
+    case BST_PEA_OCTAVE_SHIFT_DOWN2:	name = "-2";	break;
+    default:				name = " 0";	break;
+    }
+  g_string_sprintfa (gstring, "%s ", name);
+
+  /* decode instrument
+   */
+  if ((pea & BST_PEA_INSTRUMENT_MASK) == BST_PEA_INSTRUMENT_0F)
+    g_string_append (gstring, "df ");
+  else
+    g_string_sprintfa (gstring, "%02u ", (pea & BST_PEA_INSTRUMENT_MASK) >> 8);
+  
+  /* movement
+   */
+  switch (pea & BST_PEA_MOVE_MASK)
+    {
+    case BST_PEA_MOVE_NEXT:		name = "|n| | | |"; break;
+    case BST_PEA_MOVE_LEFT:		name = "|l| | | |"; break;
+    case BST_PEA_MOVE_RIGHT:		name = "|r| | | |"; break;
+    case BST_PEA_MOVE_UP:		name = "|u| | | |"; break;
+    case BST_PEA_MOVE_DOWN:		name = "|d| | | |"; break;
+    case BST_PEA_MOVE_PAGE_LEFT:	name = "| |L| | |"; break;
+    case BST_PEA_MOVE_PAGE_RIGHT:	name = "| |R| | |"; break;
+    case BST_PEA_MOVE_PAGE_UP:		name = "| |U| | |"; break;
+    case BST_PEA_MOVE_PAGE_DOWN:	name = "| |D| | |"; break;
+    case BST_PEA_MOVE_JUMP_LEFT:	name = "| | |L| |"; break;
+    case BST_PEA_MOVE_JUMP_RIGHT:	name = "| | |R| |"; break;
+    case BST_PEA_MOVE_JUMP_TOP:		name = "| | |T| |"; break;
+    case BST_PEA_MOVE_JUMP_BOTTOM:	name = "| | |B| |"; break;
+    case BST_PEA_MOVE_PREV_PATTERN:	name = "| | | |P|"; break;
+    case BST_PEA_MOVE_NEXT_PATTERN:	name = "| | | |N|"; break;
+    default:				name = "| | | | |"; break;
+    }
+  g_string_sprintfa (gstring, "%s ", name);
+
+  /* flags
+   */
+  g_string_append_c (gstring, pea & BST_PEA_WRAP_TO_NOTE	? 'B' : '.');
+  g_string_append_c (gstring, pea & BST_PEA_WRAP_TO_PATTERN	? 'P' : '.');
+  g_string_append_c (gstring, pea & BST_PEA_NOTE_RESET		? 'N' : '.');
+  g_string_append_c (gstring, pea & BST_PEA_INSTRUMENT_RESET	? 'I' : '.');
+  g_string_append_c (gstring, pea & BST_PEA_SET_INSTRUMENT_0F	? 'F' : '.');
+  g_string_append_c (gstring, pea & BST_PEA_AFFECT_BASE_OCTAVE	? 'D' : '.');
+  g_string_append_c (gstring, pea & BST_PEA_WRAP_AS_CONFIG	? 'c' : '.');
+  
+  g_string_sprintfa (gstring, "\n");
+
+  *slist_p = g_slist_prepend (*slist_p, g_strdup (gstring->str));
+  g_string_free (gstring, TRUE);
+}
+
+static gint
+list_str_cmp (gconstpointer v1,
+	      gconstpointer v2)
+{
+  const gchar *s1 = v1;
+  const gchar *s2 = v2;
+  guint l1, l2;
+
+  for (l1 = 0; s1[l1]; l1++)
+    if (s1[l1] == ' ')
+      break;
+  for (l2 = 0; s2[l2]; l2++)
+    if (s2[l2] == ' ')
+      break;
+
+  return (l1 == l2) ? strcmp (s1, s2) : l1 - l2;
+}
+
+GString*
+bst_pattern_editor_class_keydump (BstPatternEditorClass *class)
+{
+  GString *gstring;
+  GSList *node, *slist = NULL;
+  
+  g_return_val_if_fail (BST_IS_PATTERN_EDITOR_CLASS (class), NULL);
+  
+  gstring = g_string_new ("Pattern editor keytable:\n"
+			  "========================\n"
+			  "\n"
+			  "             Modifier (Shift, Control, ALt)\n"
+			  "             |   Note\n"
+			  "             |   |    Octave shift\n"
+			  "             |   |    | Instrument id (df=default instrument)\n"
+			  "             |   |    | |   Note movement (Next, Left, Right, Up, Down)\n"
+			  "             |   |    | |   | Page movement (Left, Right, Up, Down)\n"
+			  "             |   |    | |   | | Jump to border (Left, Right, Top, Bottom)\n"
+			  "             |   |    | |   | | | Switch Pattern (Next, Previous)\n"
+			  "             |   |    | |   | | | |  Flag Values:\n"
+			  "             |   |    | |   | | | |  | B - Wrap around left and right\n"
+			  "             |   |    | |   | | | |  |     border\n"
+			  "             |   |    | |   | | | |  | P - Wrap to previous/next pattern\n"
+			  "             |   |    | |   | | | |  |     at top and bottom border\n"
+			  "             |   |    | |   | | | |  | N - Reset (delete) current note\n"
+			  "             |   |    | |   | | | |  | I - Reset (delete) current instrument\n"
+			  "             |   |    | |   | | | |  | F - Set default instrument of\n"
+			  "             |   |    | |   | | | |  |     current channel\n"
+			  "             |   |    | |   | | | |  | D - Apply octave shift to the\n"
+			  "             |   |    | |   | | | |  |     default octave of the pattern\n"
+			  "KeyName      Mod Not Oc In Movement  Flags\n");
+  
+  g_hash_table_foreach (class->pea_ktab,
+			string_dump_pea,
+			&slist);
+
+  slist = g_slist_sort (slist, list_str_cmp);
+
+  for (node = slist; node; node = node->next)
+    {
+      g_string_append (gstring, node->data);
+      g_free (node->data);
+    }
+  g_slist_free (slist);
+
+  return gstring;
+}
