@@ -151,7 +151,7 @@ bse_categories_register_icon (const gchar      *category,
       if (pixdata->type && pixdata->width && pixdata->height && pixdata->encoded_pix_data)
 	{
 	  centry->icon = bse_icon_from_pixdata (pixdata); /* static reference */
-	  bse_icon_static_ref (centry->icon);
+	  bse_icon_ref_static (centry->icon);
 	}
       else
 	centry->icon = NULL;
