@@ -4,10 +4,12 @@
  **/
   { &bse_type_builtin_id_BseEffect, bse_type_builtin_register_BseEffect },
   { &bse_type_builtin_id_BseEffectFineTune, bse_type_builtin_register_BseEffectFineTune },
+  { &bse_type_builtin_id_BseItem, bse_type_builtin_register_BseItem },
+  { &bse_type_builtin_id_BsePatternGroup, bse_type_builtin_register_BsePatternGroup },
+  { &bse_type_builtin_id_BseSource, bse_type_builtin_register_BseSource },
+  { &bse_type_builtin_id_BseSInstrument, bse_type_builtin_register_BseSInstrument },
   { &bse_type_builtin_id_BseEffectVolumeDelta, bse_type_builtin_register_BseEffectVolumeDelta },
   { &bse_type_builtin_id_BseEffectPatternBreak, bse_type_builtin_register_BseEffectPatternBreak },
-  { &bse_type_builtin_id_BseItem, bse_type_builtin_register_BseItem },
-  { &bse_type_builtin_id_BseSource, bse_type_builtin_register_BseSource },
   { &bse_type_builtin_id_BseContainer, bse_type_builtin_register_BseContainer },
   { &bse_type_builtin_id_BseSuper, bse_type_builtin_register_BseSuper },
   { &bse_type_builtin_id_BseDevice, bse_type_builtin_register_BseDevice },
@@ -15,6 +17,7 @@
   { &bse_type_builtin_id_BsePcmDeviceAlsa, bse_type_builtin_register_BsePcmDeviceAlsa },
   { &bse_type_builtin_id_BseBinData, bse_type_builtin_register_BseBinData },
   { &bse_type_builtin_id_BseHeart, bse_type_builtin_register_BseHeart },
+  { &bse_type_builtin_id_BseCapture, bse_type_builtin_register_BseCapture },
   { &bse_type_builtin_id_BseSong, bse_type_builtin_register_BseSong },
   { &bse_type_builtin_id_BseProject, bse_type_builtin_register_BseProject },
   { &bse_type_builtin_id_BseEffectNoteVolume, bse_type_builtin_register_BseEffectNoteVolume },
