@@ -9,12 +9,16 @@
   { "io_changed", "BseSource", },
   { "item_added", "BseContainer", },
   { "item_removed", "BseContainer", },
+  { "cross_changes", "BseContainer", },
   { "seqid_changed", "BseItem", },
-  { "set_container", "BseItem", },
+  { "set_parent", "BseItem", },
   { "size_changed", "BsePattern", },
   { "note_changed", "BsePattern", },
-  { "note_selected", "BsePattern", },
-  { "note_unselected", "BsePattern", },
+  { "note_selection_changed", "BsePattern", },
+  { "pattern_inserted", "BsePatternGroup", },
+  { "pattern_removed", "BsePatternGroup", },
+  { "pattern_group_inserted", "BseSong", },
+  { "pattern_group_removed", "BseSong", },
   { "sequencer_step", "BseSong", },
   { "complete_restore", "BseProject", },
 
