@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1997, 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1997, 1998, 1999, 2000 Olaf Hoehmann and Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,8 +40,8 @@ enum
 static void	    bse_pattern_class_init	 (BsePatternClass	*class);
 static void	    bse_pattern_init		 (BsePattern		*pattern);
 static void	    bse_pattern_do_shutdown	 (BseObject		*object);
-static void	    bse_pattern_do_set_container (BseItem		*item,
-						  BseItem		*container);
+static void	    bse_pattern_do_set_parent    (BseItem		*item,
+						  BseItem		*parent);
 static void	    bse_pattern_reset_note	 (BsePattern		*pattern,
 						  guint			 channel,
 						  guint			 row);
@@ -70,7 +70,7 @@ BSE_BUILTIN_TYPE (BsePattern)
     NULL /* class_data */,
     
     sizeof (BsePattern),
-    16 /* n_preallocs */,
+    BSE_PREALLOC_N_PATTERNS /* n_preallocs */,
     (BseObjectInitFunc) bse_pattern_init,
   };
   
@@ -95,7 +95,7 @@ bse_pattern_class_init (BsePatternClass	*class)
   object_class->restore_private = bse_pattern_restore_private;
   object_class->shutdown = bse_pattern_do_shutdown;
   
-  item_class->set_container = bse_pattern_do_set_container;
+  item_class->set_parent = bse_pattern_do_set_parent;
 }
 
 static void
@@ -126,32 +126,25 @@ bse_pattern_do_shutdown (BseObject *object)
 }
 
 static void
-bse_pattern_do_set_container (BseItem *item,
-			      BseItem *container)
+bse_pattern_do_set_parent (BseItem *item,
+			   BseItem *parent)
 {
   BsePattern *pattern;
   
-  if (container)
-    g_return_if_fail (BSE_IS_SONG (container));
+  if (parent)
+    g_return_if_fail (BSE_IS_SONG (parent));
   
   pattern = BSE_PATTERN (item);
   
-  /* chain parent class' set_container handler */
-  BSE_ITEM_CLASS (parent_class)->set_container (item, container);
+  /* chain parent class' set_parent handler */
+  BSE_ITEM_CLASS (parent_class)->set_parent (item, parent);
   
-  if (container)
+  if (parent)
     {
-      if (BSE_IS_SONG (container))
-	{
-	  BseSong *song;
-	  
-	  song = BSE_SONG (container);
-	  
-	  bse_pattern_set_n_channels (pattern, song->n_channels);
-	  bse_pattern_set_n_rows (pattern, song->pattern_length);
-	}
-      else
-	g_warning ("BsePattern is mean to be added to containers of type BseSong only");
+      BseSong *song = BSE_SONG (parent);
+      
+      bse_pattern_set_n_channels (pattern, song->n_channels);
+      bse_pattern_set_n_rows (pattern, song->pattern_length);
     }
 }
 
@@ -160,7 +153,7 @@ bse_pattern_reset_note (BsePattern *pattern,
 			guint	    channel,
 			guint	    row)
 {
-  BseNote *note;
+  BsePatternNote *note;
   
   note = &PNOTE (pattern, channel, row);
 
@@ -171,12 +164,11 @@ bse_pattern_reset_note (BsePattern *pattern,
       note->instrument = NULL;
     }
 
-  /* leave selected */
-
   if (note->effects || note->n_effects)
     g_warning ("effects are not yet imlemented, bogus non-NULL pointer encountered");
   note->effects = NULL;
   note->n_effects = 0;
+  /* leave note->selected state */
 }
 
 void
@@ -191,14 +183,14 @@ bse_pattern_set_n_channels (BsePattern *pattern,
   
   if (pattern->n_channels != n_channels)
     {
-      BseNote *notes;
+      BsePatternNote *notes;
       guint c, r;
       
       for (c = n_channels; c < pattern->n_channels; c++)
 	for (r = 0; r < pattern->n_rows; r++)
 	  bse_pattern_reset_note (pattern, c, r);
       
-      notes = g_new (BseNote, n_channels * pattern->n_rows);
+      notes = g_new (BsePatternNote, n_channels * pattern->n_rows);
       
       for (c = 0; c < n_channels; c++)
 	for (r = 0; r < pattern->n_rows; r++)
@@ -209,8 +201,8 @@ bse_pattern_set_n_channels (BsePattern *pattern,
 	      {
 		notes[INDX (n_channels, c, r)].note = BSE_NOTE_VOID;
 		notes[INDX (n_channels, c, r)].instrument = NULL;
-		notes[INDX (n_channels, c, r)].selected = FALSE;
 		notes[INDX (n_channels, c, r)].n_effects = 0;
+		notes[INDX (n_channels, c, r)].selected = FALSE;
 		notes[INDX (n_channels, c, r)].effects = NULL;
 	      }
 	  }
@@ -235,14 +227,14 @@ bse_pattern_set_n_rows (BsePattern *pattern,
 
   if (pattern->n_rows != n_rows)
     {
-      BseNote *notes;
+      BsePatternNote *notes;
       guint c, r;
       
       for (r = n_rows; r < pattern->n_rows; r++)
 	for (c = 0; c < pattern->n_channels; c++)
 	  bse_pattern_reset_note (pattern, c, r);
       
-      notes = g_new (BseNote, pattern->n_channels * n_rows);
+      notes = g_new (BsePatternNote, pattern->n_channels * n_rows);
       
       for (c = 0; c < pattern->n_channels; c++)
 	for (r = 0; r < n_rows; r++)
@@ -253,8 +245,8 @@ bse_pattern_set_n_rows (BsePattern *pattern,
 	      {
 		notes[INDX (pattern->n_channels, c, r)].note = BSE_NOTE_VOID;
 		notes[INDX (pattern->n_channels, c, r)].instrument = NULL;
-		notes[INDX (pattern->n_channels, c, r)].selected = FALSE;
 		notes[INDX (pattern->n_channels, c, r)].n_effects = 0;
+		notes[INDX (pattern->n_channels, c, r)].selected = FALSE;
 		notes[INDX (pattern->n_channels, c, r)].effects = NULL;
 	      }
 	  }
@@ -271,11 +263,10 @@ void
 bse_pattern_modify_note (BsePattern    *pattern,
 			 guint          channel,
 			 guint          row,
-			 guint          note_val,
-			 BseInstrument *instrument,
-			 gboolean       selected)
+			 gint           note_val,
+			 BseInstrument *instrument)
 {
-  BseNote *note;
+  BsePatternNote *note;
   guint notify_changed = 0;
 
   g_return_if_fail (BSE_IS_PATTERN (pattern));
@@ -306,17 +297,6 @@ bse_pattern_modify_note (BsePattern    *pattern,
   if (notify_changed)
     BSE_NOTIFY (pattern, note_changed, NOTIFY (OBJECT, channel, row, DATA));
 
-  if (!note->selected && selected)
-    {
-      note->selected = TRUE;
-      BSE_NOTIFY (pattern, note_selected, NOTIFY (OBJECT, channel, row, DATA));
-    }
-  else if (note->selected && !selected)
-    {
-      note->selected = FALSE;
-      BSE_NOTIFY (pattern, note_unselected, NOTIFY (OBJECT, channel, row, DATA));
-    }
-
   bse_object_unlock (BSE_OBJECT (pattern));
 }
 
@@ -324,19 +304,18 @@ void
 bse_pattern_set_note (BsePattern *pattern,
 		      guint       channel,
 		      guint       row,
-		      guint       note)
+		      gint        note)
 {
+  BsePatternNote *pnote;
+
   g_return_if_fail (BSE_IS_PATTERN (pattern));
   g_return_if_fail (channel < pattern->n_channels);
   g_return_if_fail (row < pattern->n_rows);
   if (note != BSE_NOTE_VOID)
     g_return_if_fail (note >= BSE_MIN_NOTE && note <= BSE_MAX_NOTE);
 
-  bse_item_exec_void_proc (BSE_ITEM (pattern),
-			   "set-note",
-			   channel,
-			   row,
-			   note);
+  pnote = bse_pattern_peek_note (pattern, channel, row);
+  bse_pattern_modify_note (pattern, channel, row, note, pnote->instrument);
 }
 
 void
@@ -345,17 +324,16 @@ bse_pattern_set_instrument (BsePattern    *pattern,
 			    guint          row,
 			    BseInstrument *instrument)
 {
+  BsePatternNote *pnote;
+
   g_return_if_fail (BSE_IS_PATTERN (pattern));
   g_return_if_fail (channel < pattern->n_channels);
   g_return_if_fail (row < pattern->n_rows);
   if (instrument)
     g_return_if_fail (BSE_IS_INSTRUMENT (instrument));
 
-  bse_item_exec_void_proc (BSE_ITEM (pattern),
-			   "set-instrument",
-			   channel,
-			   row,
-			   instrument);
+  pnote = bse_pattern_peek_note (pattern, channel, row);
+  bse_pattern_modify_note (pattern, channel, row, pnote->note, instrument);
 }
 
 void
@@ -363,7 +341,7 @@ bse_pattern_select_note (BsePattern *pattern,
 			 guint       channel,
 			 guint       row)
 {
-  BseNote *note;
+  BsePatternNote *note;
 
   g_return_if_fail (BSE_IS_PATTERN (pattern));
   g_return_if_fail (channel < pattern->n_channels);
@@ -371,19 +349,10 @@ bse_pattern_select_note (BsePattern *pattern,
 
   note = &PNOTE (pattern, channel, row);
   if (!note->selected)
-    bse_pattern_modify_note (pattern,
-			     channel, row,
-			     note->note,
-			     note->instrument,
-			     TRUE);
-  
-  /*
-    if (!PNOTE (pattern, channel, row).selected)
-    bse_item_exec_void_proc (BSE_ITEM (pattern),
-			     "select-note",
-			     channel,
-			     row);
-  */
+    {
+      note->selected = TRUE;
+      BSE_NOTIFY (pattern, note_selection_changed, NOTIFY (OBJECT, channel, row, DATA));
+    }
 }
 
 void
@@ -391,26 +360,118 @@ bse_pattern_unselect_note (BsePattern *pattern,
 			   guint       channel,
 			   guint       row)
 {
-  BseNote *note;
+  BsePatternNote *note;
 
   g_return_if_fail (BSE_IS_PATTERN (pattern));
   g_return_if_fail (channel < pattern->n_channels);
   g_return_if_fail (row < pattern->n_rows);
-  
+
   note = &PNOTE (pattern, channel, row);
   if (note->selected)
-    bse_pattern_modify_note (pattern,
-			     channel, row,
-			     note->note,
-			     note->instrument,
-			     FALSE);
-  /*
-      if (PNOTE (pattern, channel, row).selected)
-    bse_item_exec_void_proc (BSE_ITEM (pattern),
-			     "unselect-note",
-			     channel,
-			     row);
-  */
+    {
+      note->selected = FALSE;
+      BSE_NOTIFY (pattern, note_selection_changed, NOTIFY (OBJECT, channel, row, DATA));
+    }
+}
+
+guint32*
+bse_pattern_selection_new (guint n_channels,
+			   guint n_rows)
+{
+  guint32 *selection;
+
+  g_return_val_if_fail (n_channels >= 1 && n_channels <= BSE_MAX_N_CHANNELS, NULL);
+  g_return_val_if_fail (n_rows >= 1 && n_rows <= BSE_MAX_N_ROWS, NULL);
+
+  selection = g_new0 (guint32, (n_channels * n_rows + 31) / 32 + 2);
+  BSE_PATTERN_SELECTION_N_CHANNELS (selection) = n_channels;
+  BSE_PATTERN_SELECTION_N_ROWS (selection) = n_rows;
+
+  return selection;
+}
+
+guint32*
+bse_pattern_selection_copy (guint32 *src_selection)
+{
+  guint32 *selection;
+  guint n_channels, n_rows, s;
+  
+  g_return_val_if_fail (src_selection != NULL, NULL);
+
+  n_channels = BSE_PATTERN_SELECTION_N_CHANNELS (src_selection);
+  n_rows = BSE_PATTERN_SELECTION_N_ROWS (src_selection);
+  s = (n_channels * n_rows + 31) / 32 + 2;
+  selection = g_new (guint32, s);
+  memcpy (selection, src_selection, sizeof (guint32) * s);
+
+  return selection;
+}
+
+void
+bse_pattern_selection_free (guint32 *selection)
+{
+  g_return_if_fail (selection != NULL);
+
+  g_free (selection);
+}
+
+void
+bse_pattern_selection_fill (guint32 *selection,
+			    gboolean selected)
+{
+  guint n;
+
+  g_return_if_fail (selection != NULL);
+
+  n = BSE_PATTERN_SELECTION_N_CHANNELS (selection) * BSE_PATTERN_SELECTION_N_ROWS (selection);
+  memset (selection + 2, selected ? 0xff : 0, (n / 32) * sizeof (guint32));
+}
+
+void
+bse_pattern_save_selection (BsePattern *pattern,
+			    guint32    *selection)
+{
+  guint c, r;
+
+  g_return_if_fail (BSE_IS_PATTERN (pattern));
+  g_return_if_fail (selection != NULL);
+  g_return_if_fail (BSE_PATTERN_SELECTION_N_CHANNELS (selection) == pattern->n_channels);
+  g_return_if_fail (BSE_PATTERN_SELECTION_N_ROWS (selection) == pattern->n_rows);
+
+  for (c = 0; c < pattern->n_channels; c++)
+    for (r = 0; r < pattern->n_rows; r++)
+      {
+	BsePatternNote *note = &PNOTE (pattern, c, r);
+
+	if (note->selected)
+	  BSE_PATTERN_SELECTION_MARK (selection, c, r);
+	else
+	  BSE_PATTERN_SELECTION_UNMARK (selection, c, r);
+      }
+}
+
+void
+bse_pattern_restore_selection (BsePattern *pattern,
+			       guint32    *selection)
+{
+  guint c, r;
+
+  g_return_if_fail (BSE_IS_PATTERN (pattern));
+  g_return_if_fail (selection != NULL);
+  g_return_if_fail (BSE_PATTERN_SELECTION_N_CHANNELS (selection) == pattern->n_channels);
+  g_return_if_fail (BSE_PATTERN_SELECTION_N_ROWS (selection) == pattern->n_rows);
+
+  for (c = 0; c < pattern->n_channels; c++)
+    for (r = 0; r < pattern->n_rows; r++)
+      {
+	BsePatternNote *note = &PNOTE (pattern, c, r);
+	gboolean selected = BSE_PATTERN_SELECTION_TEST (selection, c, r);
+
+	if (note->selected && !selected)
+	  bse_pattern_unselect_note (pattern, c, r);
+	else if (!note->selected && selected)
+	  bse_pattern_select_note (pattern, c, r);
+      }
 }
 
 GList* /* free list */
@@ -424,7 +485,7 @@ bse_pattern_list_selection (BsePattern *pattern)
   for (c = 0; c < pattern->n_channels; c++)
     for (r = 0; r < pattern->n_rows; r++)
       {
-	BseNote *note = &PNOTE (pattern, c, r);
+	BsePatternNote *note = &PNOTE (pattern, c, r);
 
 	if (note->selected)
 	  list = g_list_prepend (list, note);
@@ -433,7 +494,25 @@ bse_pattern_list_selection (BsePattern *pattern)
   return g_list_reverse (list);
 }
 
-BseNote*
+gboolean
+bse_pattern_has_selection (BsePattern *pattern)
+{
+  guint c, r;
+
+  g_return_val_if_fail (BSE_IS_PATTERN (pattern), FALSE);
+
+  for (c = 0; c < pattern->n_channels; c++)
+    for (r = 0; r < pattern->n_rows; r++)
+      {
+	BsePatternNote *note = &PNOTE (pattern, c, r);
+
+	if (note->selected)
+	  return TRUE;
+      }
+  return FALSE;
+}
+
+BsePatternNote*
 bse_pattern_peek_note (BsePattern *pattern,
 		       guint       channel,
 		       guint       row)
@@ -445,12 +524,12 @@ bse_pattern_peek_note (BsePattern *pattern,
   return &PNOTE (pattern, channel, row);
 }
 
-static BseNote*
+static BsePatternNote*
 get_note (BsePattern *pattern,
 	  guint	      channel,
 	  guint	      row)
 {
-  BseNote *note;
+  BsePatternNote *note;
   
   while (row >= pattern->n_rows)
     {
@@ -473,8 +552,8 @@ get_note (BsePattern *pattern,
 }
 
 static void
-save_note (BseStorage *storage,
-	   BseNote    *note)
+save_note (BseStorage     *storage,
+	   BsePatternNote *note)
 {
   bse_storage_break (storage);
   bse_storage_putc (storage, '(');
@@ -526,7 +605,7 @@ bse_pattern_store_private (BseObject  *object,
     {
       for (r = 0; r < pattern->n_rows; r++)
 	{
-	  BseNote *note, *note2, *note3;
+	  BsePatternNote *note, *note2, *note3;
 	  
 	  note = get_note (pattern, c, r);
 	  note2 = get_note (pattern, c, r + 1);
@@ -574,7 +653,7 @@ bse_pattern_restore_private (BseObject	*object,
   static GQuark quark_move_channel = 0;
   static GQuark quark_move_row = 0;
   GQuark token_quark;
-  guint note = BSE_NOTE_VOID;
+  gint note = BSE_NOTE_VOID;
   BseInstrument *instrument = NULL;
   BsePattern *pattern;
   
@@ -752,7 +831,8 @@ bse_pattern_restore (BseObject	*object,
   
   /* chain parent class' handler */
   expected_token = BSE_OBJECT_CLASS (parent_class)->restore (object, storage);
-  
+
+  /* clean up, parsing is done */
   pattern->current_channel = current_channel;
   pattern->current_row = current_row;
   
