@@ -25,6 +25,8 @@ BSE_DUMMY_TYPE (BsePcmDeviceOSS);
 
 #include	<sys/soundcard.h>
 #include	<sys/ioctl.h>
+#include	<sys/types.h>
+#include	<sys/time.h>
 #include	<unistd.h>
 #include	<errno.h>
 #include	<fcntl.h>
@@ -47,7 +49,7 @@ struct _BsePcmDeviceOSSClass
 static void	    bse_pcm_device_oss_class_init	(BsePcmDeviceOSSClass	*class);
 static void	    bse_pcm_device_oss_init		(BsePcmDeviceOSS	*pcm_device_oss);
 static void	    bse_pcm_device_oss_shutdown		(BseObject		*object);
-static gchar*	    bse_pcm_device_oss_device_name	(BsePcmDevice           *pdev,
+static gchar*	    bse_pcm_device_oss_device_name	(BseDevice              *dev,
 							 gboolean                descriptive);
 static BseErrorType bse_pcm_device_oss_update_caps	(BsePcmDevice		*pdev);
 static BseErrorType bse_pcm_device_oss_open		(BsePcmDevice		*pdev,
@@ -65,8 +67,7 @@ static BseErrorType bse_pcm_device_oss_setup		(BsePcmDevice		*pdev,
 							 guint          	 fragment_size);
 static void	    bse_pcm_device_oss_retrigger	(BsePcmDevice		*pdev);
 static void	    bse_pcm_device_oss_update_state	(BsePcmDevice   	*pdev);
-static gboolean	    bse_pcm_device_oss_in_playback	(BsePcmDevice		*pdev);
-static void	    bse_pcm_device_oss_close		(BsePcmDevice		*pdev);
+static void	    bse_pcm_device_oss_close		(BseDevice		*pdev);
 
 
 /* --- variables --- */
@@ -105,21 +106,23 @@ static void
 bse_pcm_device_oss_class_init (BsePcmDeviceOSSClass *class)
 {
   BseObjectClass *object_class;
+  BseDeviceClass *device_class;
   BsePcmDeviceClass *pcm_device_class;
   
   parent_class = bse_type_class_peek (BSE_TYPE_PCM_DEVICE);
   object_class = BSE_OBJECT_CLASS (class);
+  device_class = BSE_DEVICE_CLASS (class);
   pcm_device_class = BSE_PCM_DEVICE_CLASS (class);
   
   object_class->shutdown = bse_pcm_device_oss_shutdown;
   
-  pcm_device_class->device_name = bse_pcm_device_oss_device_name;
+  device_class->close = bse_pcm_device_oss_close;
+  device_class->device_name = bse_pcm_device_oss_device_name;
+
   pcm_device_class->update_caps = bse_pcm_device_oss_update_caps;
   pcm_device_class->open = bse_pcm_device_oss_open;
   pcm_device_class->retrigger = bse_pcm_device_oss_retrigger;
   pcm_device_class->update_state = bse_pcm_device_oss_update_state;
-  pcm_device_class->in_playback = bse_pcm_device_oss_in_playback;
-  pcm_device_class->close = bse_pcm_device_oss_close;
 }
 
 static void
@@ -144,10 +147,10 @@ bse_pcm_device_oss_shutdown (BseObject *object)
 }
 
 static gchar*
-bse_pcm_device_oss_device_name (BsePcmDevice *pdev,
-				gboolean      descriptive)
+bse_pcm_device_oss_device_name (BseDevice *dev,
+				gboolean   descriptive)
 {
-  BsePcmDeviceOSS *oss = BSE_PCM_DEVICE_OSS (pdev);
+  BsePcmDeviceOSS *oss = BSE_PCM_DEVICE_OSS (dev);
 
   return oss->device_name;
 }
@@ -161,6 +164,7 @@ bse_pcm_device_oss_open (BsePcmDevice  *pdev,
 			 guint          fragment_size)
 {
   BsePcmDeviceOSS *oss = BSE_PCM_DEVICE_OSS (pdev);
+  BseDevice *dev = BSE_DEVICE (oss);
   BseErrorType error;
   gint omode = 0;
   gint fd;
@@ -182,7 +186,7 @@ bse_pcm_device_oss_open (BsePcmDevice  *pdev,
       else
 	return BSE_ERROR_DEVICE_IO;
     }
-  pdev->pfd.fd = fd;
+  dev->pfd.fd = fd;
   
   error = bse_pcm_device_oss_setup (pdev, writable, readable, n_channels, rate, 128, fragment_size);
   if (error)
@@ -190,49 +194,83 @@ bse_pcm_device_oss_open (BsePcmDevice  *pdev,
   else
     {
       if (readable)
-	BSE_OBJECT_SET_FLAGS (oss, BSE_PCM_FLAG_READABLE);
+	BSE_OBJECT_SET_FLAGS (oss, BSE_DEVICE_FLAG_READABLE);
       if (writable)
-	BSE_OBJECT_SET_FLAGS (oss, BSE_PCM_FLAG_WRITABLE);
+	BSE_OBJECT_SET_FLAGS (oss, BSE_DEVICE_FLAG_WRITABLE);
     }
 
   return error;
 }
 
 static void
-bse_pcm_device_oss_close (BsePcmDevice *pdev)
+bse_pcm_device_oss_close (BseDevice *dev)
 {
-  (void) ioctl (pdev->pfd.fd, SNDCTL_DSP_RESET);
+  (void) ioctl (dev->pfd.fd, SNDCTL_DSP_RESET);
   
-  (void) close (pdev->pfd.fd);
+  (void) close (dev->pfd.fd);
+
+  /* chain parent class' handler */
+  BSE_DEVICE_CLASS (parent_class)->close (dev);
 }
 
 static void
 bse_pcm_device_oss_retrigger (BsePcmDevice *pdev)
 {
+  BseDevice *dev = BSE_DEVICE (pdev);
   int d_int;
 
-  /* historically, it used to be enough to select() on the
-   * fd to trigger capture/playback, but with some new OSS
-   * drivers (clones) this is not the case anymore, so we
+  /* it should be enough to select() on the fd to trigger
+   * capture/playback, but with some new OSS drivers
+   * (clones) this is not the case anymore, so we also
    * use the SNDCTL_DSP_SETTRIGGER ioctl to achive this.
    */
-
+  
   d_int = 0;
-  if (BSE_PCM_DEVICE_READABLE (pdev))
+  if (BSE_DEVICE_READABLE (pdev))
     d_int |= PCM_ENABLE_INPUT;
-  if (BSE_PCM_DEVICE_WRITABLE (pdev))
+  if (BSE_DEVICE_WRITABLE (pdev))
     d_int |= PCM_ENABLE_OUTPUT;
-  (void) ioctl (pdev->pfd.fd, SNDCTL_DSP_SETTRIGGER, &d_int);
+  (void) ioctl (dev->pfd.fd, SNDCTL_DSP_SETTRIGGER, &d_int);
+  
+  /* Jaroslav Kysela <perex@jcu.cz>:
+   *  Important sequence:
+   *     1) turn on capture
+   *     2) write at least one fragment to create appropriate delay between
+   *        capture and playback; we need some time for process wakeup and
+   *        data copy in the user space
+   * e.g.:
+   * we write two fragments to playback
+   * software latency is:
+   *   2 * frag_size + delay between capture start & playback start
+   *
+   * TIMJ:
+   *   we only need to asure that input is actually triggered, BseHeart
+   *   takes care of the latency and at this point probably already has
+   *   output buffers pending.
+   */
+  if (BSE_DEVICE_READABLE (pdev))
+    {
+      struct timeval tv = { 0, 0, };
+      fd_set in_fds;
+      fd_set out_fds;
+
+      FD_ZERO (&in_fds);
+      FD_ZERO (&out_fds);
+      FD_SET (dev->pfd.fd, &in_fds);
+      /* FD_SET (dev->pfd.fd, &out_fds); */
+      select (dev->pfd.fd + 1, &in_fds, &out_fds, NULL, &tv);
+    }
 }
 
 static void
 bse_pcm_device_oss_update_state (BsePcmDevice *pdev)
 {
   BsePcmDeviceOSS *oss = BSE_PCM_DEVICE_OSS (pdev);
+  BseDevice *dev = BSE_DEVICE (oss);
   audio_buf_info info;
   
   memset (&info, 0, sizeof (info));
-  (void) ioctl (pdev->pfd.fd, SNDCTL_DSP_GETISPACE, &info);
+  (void) ioctl (dev->pfd.fd, SNDCTL_DSP_GETISPACE, &info);
   pdev->capture_buffer_size = info.fragstotal * info.fragsize;
   pdev->n_capture_bytes = info.fragments * info.fragsize;
   pdev->n_capture_bytes = info.bytes;
@@ -251,7 +289,7 @@ bse_pcm_device_oss_update_state (BsePcmDevice *pdev)
 	       info.bytes);
   
   memset (&info, 0, sizeof (info));
-  (void) ioctl (pdev->pfd.fd, SNDCTL_DSP_GETOSPACE, &info);
+  (void) ioctl (dev->pfd.fd, SNDCTL_DSP_GETOSPACE, &info);
   pdev->playback_buffer_size = info.fragstotal * info.fragsize;
   pdev->n_playback_bytes = info.fragments * info.fragsize;
   pdev->n_playback_bytes = info.bytes;
@@ -270,16 +308,6 @@ bse_pcm_device_oss_update_state (BsePcmDevice *pdev)
 	       info.bytes);
 }
 
-static gboolean
-bse_pcm_device_oss_in_playback (BsePcmDevice *pdev)
-{
-  audio_buf_info info = { 0, };
-  
-  (void) ioctl (pdev->pfd.fd, SNDCTL_DSP_GETOSPACE, &info);
-  
-  return info.fragments < info.fragstotal;
-}
-
 static BseErrorType
 bse_pcm_device_oss_setup (BsePcmDevice	*pdev,
 			  gboolean       writable,
@@ -290,7 +318,8 @@ bse_pcm_device_oss_setup (BsePcmDevice	*pdev,
 			  guint          fragment_size)
 {
   BsePcmDeviceOSS *oss = BSE_PCM_DEVICE_OSS (pdev);
-  gint fd = pdev->pfd.fd;
+  BseDevice *dev = BSE_DEVICE (oss);
+  gint fd = dev->pfd.fd;
   glong d_long;
   gint d_int;
   
